<?php

namespace Drupal\paynl_payment\Plugin\Payment\Method;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\payment\EventDispatcherInterface;
use Drupal\payment\Plugin\Payment\Status\PaymentStatusManagerInterface;
use Drupal\paynl_payment\Helpers\PaynlSdkHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\payment\OperationResult;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodBase;
use Drupal\payment\Response\Response;

/**
 * Class PaynlPayment.
 *
 * @PaymentMethod(
 *   deriver = "Drupal\paynl_payment\Plugin\Payment\Method\PaynlPaymentDeriver",
 *   id = "paynl_payment",
 *   operations_provider = "\Drupal\paynl_payment\Plugin\Payment\Method\PaynlPaymentOperationsProvider",
 * )
 */
class PaynlPayment extends PaymentMethodBase {

  /**
   * Pay.nl SDK helper.
   *
   * @var \Drupal\paynl_payment\Helpers\PaynlSdkHelper
   */
  protected $paynlSdkHelper;

  /**
   * Messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The redirect URL for the off site payment.
   *
   * @var null|\Drupal\Core\Url
   */
  protected $redirectUrl = NULL;

  /**
   * Constructs a new instance.
   *
   * @param mixed[] $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed[] $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\payment\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\Core\Utility\Token $token
   *   The token API.
   * @param \Drupal\payment\Plugin\Payment\Status\PaymentStatusManagerInterface $payment_status_manager
   *   The payment status manager.
   * @param \Drupal\paynl_payment\Helpers\PaynlSdkHelper $paynlSdkHelper
   *   Pay.nl SDK helper.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Messenger.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    array $plugin_definition,
    ModuleHandlerInterface $module_handler,
    EventDispatcherInterface $event_dispatcher,
    Token $token,
    PaymentStatusManagerInterface $payment_status_manager,
    PaynlSdkHelper $paynlSdkHelper,
    MessengerInterface $messenger
  ) {
    $configuration += $this->defaultConfiguration();
    $this->paynlSdkHelper = $paynlSdkHelper;
    $this->messenger = $messenger;
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $module_handler,
      $event_dispatcher,
      $token,
      $payment_status_manager
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler'),
      $container->get('payment.event_dispatcher'),
      $container->get('token'),
      $container->get('plugin.manager.payment.status'),
      $container->get('paynl_payment.helper'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedCurrencies() {
    // Returning true allows all currencies.
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getPaymentExecutionResult() {
    if ($this->redirectUrl) {
      $redirectUrl = $this->redirectUrl;
    }
    else {
      // Something went wrong. Redirect to the current page.
      $redirectUrl = Url::fromRoute('<current>');
    }

    return new OperationResult(new Response($redirectUrl));
  }

  /**
   * {@inheritdoc}
   */
  protected function doExecutePayment() {
    try {
      $payment = $this->getPayment();

      // Get line items to pass them to Pay.nl.
      // Todo: determine the tax amount.
      $line_items = [];
      foreach ($payment->getLineItems() as $line_item) {
        $line_items[] = [
          'id' => $line_item->getName(),
          'name' => $line_item->getName(),
          'price' => $line_item->getAmount(),
          'tax' => $line_item->getAmount() - $line_item->getAmount(),
          'qty' => $line_item->getQuantity(),
        ];
      }

      try {
        // Start transaction.
        $options = [
          'amount' => $payment->getAmount(),
          'returnUrl' => Url::fromRoute(
            'paynl_payment.return',
            ['payment' => $payment->id()],
            ['absolute' => TRUE]
          ),
          'exchangeUrl' => Url::fromRoute(
            'paynl_payment.exchange',
            ['payment' => $payment->id()],
            ['absolute' => TRUE]
          ),
          'description' => $payment->id(),
          'products' => $line_items,
          'language' => strtoupper(\Drupal::languageManager()->getCurrentLanguage()->getId()),
          'ipaddress' => \Drupal::request()->getClientIp(),
          'testmode' => $this->paynlSdkHelper->hasTestMode(),
        ];
        $transaction = $this->paynlSdkHelper->startTransaction($options);

        // Set redirect URL for off site payment.
        $this->redirectUrl = Url::fromUri($transaction->getRedirectUrl());
      }
      catch (\Exception $e) {
        $this->paynlSdkHelper->logException($e);
      }
    }
    catch (\Exception $e) {
      $this->paynlSdkHelper->logException($e);
    }
  }

}
